@@ -30,12 +30,8 @@ int state = STATE_NONAUTH;      /* Current IMAP4 state */
 struct mu_auth_data *auth_data; 
 
 int login_disabled;             /* Disable LOGIN command */
-int create_home_dir;            /* Create home directory if it does not
-				   exist */
 mu_list_t user_retain_groups;
 
-int home_dir_mode = S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH;
-
 /* Saved command line. */
 int imap4d_argc;                 
 char **imap4d_argv;
@@ -123,6 +119,7 @@ static struct mu_option imap4d_options[] = {
 
 static char *capa[] = {
   "auth",
+  "homedir",
   "debug",
   "mailbox",
   "locking",
@@ -130,18 +127,6 @@ static char *capa[] = {
   NULL
 };
 
-static int
-cb_mode (void *data, mu_config_value_t *val)
-{
-  char *p;
-  if (mu_cfg_assert_value_type (val, MU_CFG_STRING))
-    return 1;
-  home_dir_mode = strtoul (val->v.string, &p, 8);
-  if (p[0] || (home_dir_mode & ~0777))
-    mu_error (_("invalid mode specification: %s"), val->v.string);
-  return 0;
-}
-
 int
 parse_preauth_scheme (const char *scheme, mu_url_t url)
 {
@@ -340,7 +325,37 @@ cb_prefix_delim (void *data, mu_config_value_t *val)
     *(int*) data = val->v.string[0];
   return 0;
 }
+
+static int
+cb_create_home_dir (void *data, mu_config_value_t *val)
+{
+  mu_diag_output (MU_DIAG_WARNING, _("%s is deprecated; use %s instead"),
+		  "create-home-dir", "homedir.create");
+  
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING))
+    return 1;
 
+  return mu_str_to_c (val->v.string, mu_c_bool, &mu_homedir_create, NULL);
+}
+
+static int
+cb_home_dir_mode (void *data, mu_config_value_t *val)
+{
+  int n;
+  char *p;
+
+  mu_diag_output (MU_DIAG_WARNING, _("%s is deprecated; use %s instead"),
+		  "home-dir-mode", "homedir.mode");
+
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING))
+    return 1;
+  n = strtoul (val->v.string, &p, 8);
+  if (p[0] || (n & ~0777))
+    mu_error (_("invalid mode specification: %s"), val->v.string);
+  mu_homedir_mode = n;
+  return 0;
+}
+
 static int
 cb_prefix_scheme (void *data, mu_config_value_t *val)
 {
@@ -533,10 +548,10 @@ static struct mu_cfg_param imap4d_cfg_param[] = {
   
   { "login-disabled", mu_c_bool, &login_disabled, 0, NULL,
     N_("Disable LOGIN command.") },
-  { "create-home-dir", mu_c_bool, &create_home_dir, 0, NULL,
-    N_("If true, create non-existing user home directories.") },
-  { "home-dir-mode", mu_cfg_callback, NULL, 0, cb_mode,
-    N_("File mode for creating user home directories."),
+  { "create-home-dir", mu_cfg_callback, NULL, 0, cb_create_home_dir,
+    N_("DEPRECATED.  Use homedir.create instead.") },
+  { "home-dir-mode", mu_cfg_callback, NULL, 0, cb_home_dir_mode,
+    N_("DEPRECATED.  Use homedir.mode instead."),
     N_("mode: octal") },
   { "retain-groups", mu_cfg_callback, &user_retain_groups, 0, cb_group,
     N_("Retain these supplementary groups when switching to user privileges"),
@@ -547,7 +562,7 @@ static struct mu_cfg_param imap4d_cfg_param[] = {
     N_("Kind of TLS encryption to use for the inetd server"
        " and all server blocks that lack the tls-mode statement."),
     /* TRANSLATORS: words to the right of : are keywords - do not translate */
-    N_("arg: false|true|ondemand|stls|requred|connection") },
+    N_("arg: false|true|ondemand|stls|required|connection") },
   { "preauth", mu_cfg_callback, NULL, 0, cb_preauth,
     N_("Configure PREAUTH mode.  <value> is one of:\n"
        "  prog:///<full-program-name: string>\n"
@@ -626,6 +641,8 @@ mu_get_user_groups (const char *user, mu_list_t retain, mu_list_t *pgrouplist)
 int
 imap4d_session_setup0 ()
 {
+  int rc;
+
   if (auth_deny_user_list &&
       mu_list_locate (auth_deny_user_list, auth_data->name, NULL) == 0)
     {
@@ -652,8 +669,14 @@ imap4d_session_setup0 ()
     }
 
   real_homedir = mu_normalize_path (mu_strdup (auth_data->dir));
-  if (imap4d_check_home_dir (real_homedir, auth_data->uid, auth_data->gid))
-    return 1;
+  rc = mu_homedir_assert (real_homedir, auth_data->uid, auth_data->gid);
+  if (rc)
+    {
+      mu_error (_("can't set up user home directory %s: %s"), real_homedir,
+		mu_strerror (rc));
+      free (real_homedir);
+      return 1;
+    }
   
   if (auth_data->change_uid)
     {
@@ -836,36 +859,6 @@ imap4d_connection (int fd, struct sockaddr *sa, int salen,
   return 0;
 }
 
-int
-imap4d_check_home_dir (const char *dir, uid_t uid, gid_t gid)
-{
-  struct stat st;
-
-  if (stat (dir, &st))
-    {
-      if (errno == ENOENT && create_home_dir)
-	{
-	  mode_t mode = umask (0);
-	  int rc = mkdir (dir, home_dir_mode);
-	  umask (mode);
-	  if (rc)
-	    {
-	      mu_error ("Cannot create home directory `%s': %s",
-			dir, mu_strerror (errno));
-	      return 1;
-	    }
-	  if (chown (dir, uid, gid))
-	    {
-	      mu_error ("Cannot set owner for home directory `%s': %s",
-			dir, mu_strerror (errno));
-	      return 1;
-	    }
-	}
-    }
-  
-  return 0;
-}
-
 
 jmp_buf master_jmp;
 
