@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2011-2019 Free Software Foundation, Inc.
+   Copyright (C) 2011-2025 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -37,14 +37,14 @@ struct list_closure
 };
 
 static int
-count_level (const char *name, int delim)
+count_depth (const char *name, int delim)
 {
-  int level = 0;
+  int depth = 0;
 
   while (*name)
     if (*name++ == delim)
-      level++;
-  return level;
+      depth++;
+  return depth;
 }
 
 static int
@@ -78,7 +78,8 @@ list_untagged_handler (mu_imap_t imap, mu_list_t resp, void *data)
       _mu_imap_list_nth_element_is_string (resp, 0, clos->command))
     {
       struct mu_list_response *rp;
-
+      char const *name;
+      
       rp = calloc (1, sizeof (*rp));
       if (!rp)
 	{
@@ -95,7 +96,31 @@ list_untagged_handler (mu_imap_t imap, mu_list_t resp, void *data)
       elt = _mu_imap_list_at (resp, 3);
       if (!(elt && elt->type == imap_eltype_string))
 	return;
-      rp->name = strdup (elt->v.string);
+
+      name = elt->v.string;
+      /*
+       * Skip first imap->prefix_len characters of the returned name.  This
+       * is the length of the path part in the folder URL.  Note, however,
+       * that there is a case when an IMAP server can return a name shorter
+       * than that: namely, when given an empty mailbox (wref) name.  As per
+       * RFC 3501:
+       *
+       *   An empty ("" string) mailbox name argument is a special request to
+       *   return the hierarchy delimiter and the root name of the name given
+       *   in the reference.
+       *
+       * The following conditional takes this into account.  It is based on
+       * a reasonable assumption that the root name is always shorter than
+       * the reference name.
+       *
+       * FIXME: Perhaps it would be safer to compare with the pathname prefix
+       * instead.  To do so, mu_imap_t should include the prefix (or a pointer
+       * to mu_folder_t).
+       */
+      if (strlen (name) > imap->prefix_len)
+	name += imap->prefix_len;
+      
+      rp->name = strdup (name);
       if (!rp->name)
 	{
 	  free (rp);
@@ -109,14 +134,14 @@ list_untagged_handler (mu_imap_t imap, mu_list_t resp, void *data)
       if (_mu_imap_list_element_is_nil (elt))
 	{
 	  rp->separator = 0;
-	  rp->level = 0;
+	  rp->depth = 0;
 	}
       else if (elt->type != imap_eltype_string)
 	return;
       else
 	{
 	  rp->separator = elt->v.string[0];
-	  rp->level = count_level (rp->name, rp->separator);
+	  rp->depth = count_depth (rp->name, rp->separator);
 	}
       if ((clos->error_code = mu_list_append (clos->retlist, rp)))
 	mu_list_response_free (rp);
