@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003-2024 Free Software Foundation, Inc.
+   Copyright (C) 2003-2025 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -144,7 +144,8 @@ static struct mu_option options[] = {
     N_("don't sort on the date field"),
     mu_c_string, NULL, rem_datefield },
   { "limit",         0, N_("DAYS"), MU_OPTION_DEFAULT,
-    N_("consider two datefields equal if their difference lies within the given nuber of DAYS."),
+    N_("consider two datefields equal if their difference falls within"
+       " the given number of DAYS."),
     mu_c_int, &limit },
   { "nolimit",       0,       NULL,       MU_OPTION_DEFAULT,
     N_("undo the effect of the last -limit option"),
@@ -557,9 +558,9 @@ sort (void)
     }
   if (action == ACTION_REORDER)
     {
-      mu_mailbox_uidvalidity_reset (mbox);
       mu_mailbox_close (mbox);
       mu_mailbox_open (mbox, MU_STREAM_RDWR);
+      mu_mailbox_uidvalidity_reset (mbox);
       mh_mailbox_set_cur (mbox, current_num);
     }
 }
